unit CpuUsage;
interface

uses windows,messages,Sysutils;

type

T_WinVersion=(wv_None,wv_3X,wv_9X,wv_NT);

T_CpuUsage=Class
private
  FWinVersion:T_WinVersion;
  FActive:boolean;
  F_NT_LastValue, F_NT_Value: comp;
  F_NT_LastTicks: Cardinal;
  function GetUsage: integer;
  function GetNTCPUUsage: comp;
  function GetReestrData(Root: HKey; Key, Parameter: String): string;
  procedure ReleaseNTCPUData;
  function InitNTCPUData: Boolean;
protected
public
  function GetWinVersion:T_WinVersion;
  constructor Create;
  destructor Destroy; override;
published
  property Active:boolean read FActive;
  property Usage:integer read GetUsage;
end;



implementation

type
PULONG = ^ULONG;
ULONG = DWORD;
NTSTATUS = ULONG;
PVOID = Pointer;
_SYSTEM_INFORMATION_CLASS = (
     SystemBasicInformation,
     SystemProcessorInformation,
     SystemPerformanceInformation,
     SystemTimeOfDayInformation,
     SystemNotImplemented1,
     SystemProcessesAndThreadsInformation,
     SystemCallCounts,
     SystemConfigurationInformation,
     SystemProcessorTimes,
     SystemGlobalFlag,
     SystemNotImplemented2,
     SystemModuleInformation,
     SystemLockInformation,
     SystemNotImplemented3,
     SystemNotImplemented4,
     SystemNotImplemented5,
     SystemHandleInformation,
     SystemObjectInformation,
     SystemPagefileInformation,
     SystemInstructionEmulationCounts,
     SystemInvalidInfoClass1,
     SystemCacheInformation,
     SystemPoolTagInformation,
     SystemProcessorStatistics,
     SystemDpcInformation,
     SystemNotImplemented6,
     SystemLoadImage,
     SystemUnloadImage,
     SystemTimeAdjustment,
     SystemNotImplemented7,
     SystemNotImplemented8,
     SystemNotImplemented9,
     SystemCrashDumpInformation,
     SystemExceptionInformation,
     SystemCrashDumpStateInformation,
     SystemKernelDebuggerInformation,
     SystemContextSwitchInformation,
     SystemRegistryQuotaInformation,
     SystemLoadAndCallImage,
     SystemPrioritySeparation,
     SystemNotImplemented10,
     SystemNotImplemented11,
     SystemInvalidInfoClass2,
     SystemInvalidInfoClass3,
     SystemTimeZoneInformation,
     SystemLookasideInformation,
     SystemSetTimeSlipEvent,
     SystemCreateSession,
     SystemDeleteSession,
     SystemInvalidInfoClass4,
     SystemRangeStartInformation,
     SystemVerifierInformation,
     SystemAddVerifier,
     SystemSessionProcessesInformation);
   SYSTEM_INFORMATION_CLASS = _SYSTEM_INFORMATION_CLASS;

   _SYSTEM_PROCESSOR_TIMES = packed record
    IdleTime,
    KernelTime,
    UserTime,
    DpcTime,
    InterruptTime: int64;
    InterruptCount: ULONG;
   end;

   SYSTEM_PROCESSOR_TIMES = _SYSTEM_PROCESSOR_TIMES;
   PSYSTEM_PROCESSOR_TIMES = ^_SYSTEM_PROCESSOR_TIMES;

   TNativeQuerySystemInformation = function(
     SystemInformationClass: SYSTEM_INFORMATION_CLASS;
     SystemInformation: PVOID;
     SystemInformationLength: ULONG;
     ReturnLength: PULONG
     ): NTSTATUS; stdcall;

const
NTDLL_DLL_Name = 'NTDLL.DLL';
STATUS_SUCCESS = $00000000;
STATUS_INFO_LENGTH_MISMATCH = $C0000004;
Timer100N = 10000000;
Timer1S = 1000;

var
CPUSize: DWORD;
CPUNTUsage: PSYSTEM_PROCESSOR_TIMES;
NTDLL_DLL: THandle = 0;
ZwQuerySystemInformation: TNativeQuerySystemInformation = nil;

function T_CpuUsage.InitNTCPUData: Boolean;
var
R: NTSTATUS;
n: DWORD;
begin
n:=0;
CPUNTUsage:=AllocMem(SizeOf(SYSTEM_PROCESSOR_TIMES));
R:=ZwQuerySystemInformation(SystemProcessorTimes,CPUNTUsage,SizeOf(SYSTEM_PROCESSOR_TIMES),nil);
while R=STATUS_INFO_LENGTH_MISMATCH do begin
  Inc(n);
  ReallocMem(CPUNTUsage,n*SizeOf(CPUNTUsage^));
  R:=ZwQuerySystemInformation(SystemProcessorTimes,CPUNTUsage,n*SizeOf(SYSTEM_PROCESSOR_TIMES),nil);
end;
CPUSize:=n*SizeOf(CPUNTUsage^);
Result:=R=STATUS_SUCCESS;
end;

function T_CpuUsage.GetNTCPUUsage: comp;
begin
ZwQuerySystemInformation(SystemProcessorTimes,CPUNTUsage,CPUSize,nil);
Result:=CPUNTUsage^.IdleTime;
end;

procedure T_CpuUsage.ReleaseNTCPUData;
begin
Freemem(CPUNTUsage);
end;

function T_CpuUsage.GetReestrData(Root:HKey;Key,Parameter:String):string;
var ValueKey: HKey;
  MaxValueLen: DWORD;
  ValueBuf: LPSTR;
  I: Integer;
  Code: Integer;
  ValType:DWord;
begin
Result:='';
if RegOpenKeyEx(Root,PChar(Key),0,KEY_READ,ValueKey)=ERROR_SUCCESS then
begin
  if (RegQueryInfoKey(ValueKey,nil,nil,nil,nil,nil,nil,nil,nil,@MaxValueLen,nil,nil)=ERROR_SUCCESS) and (MaxValueLen>0) then begin
   Inc(MaxValueLen);
   repeat
    ValueBuf:=StrAlloc(MaxValueLen);
    Code:=RegQueryValueEx(ValueKey,PChar(Parameter),nil,@ValType,PByte(ValueBuf),@MaxValueLen);
    if Code=ERROR_MORE_DATA then begin
     StrDispose(ValueBuf);
     Inc(MaxValueLen);
    end;
   until (Code<>ERROR_MORE_DATA);
   if (Code=ERROR_SUCCESS) then begin
    if ValType=REG_SZ then Dec(MaxValueLen);
    for I:=0 to MaxValueLen-1 do Result:=Result+ValueBuf[I];
   end;
   StrDispose(ValueBuf);
  end;
  RegCloseKey(ValueKey);
end;
end;

{ CpyUsage }

constructor T_CpuUsage.Create;
begin
FWinVersion:=GetWinVersion;
case FWinVersion of
wv_9X:
  begin;
   FActive:=True; 
   GetReestrData(HKEY_DYN_DATA,'PerfStats\StartStat','KERNEL\CPUUsage');
  end; 
wv_NT:
  begin;
   if NTDLL_DLL=0 then NTDLL_DLL:=GetModuleHandle(NTDLL_DLL_name);
   if NTDLL_DLL<>0 then @ZwQuerySystemInformation:=GetProcAddress(NTDLL_DLL,'ZwQuerySystemInformation');
   FActive:=InitNTCPUData;
   F_NT_LastTicks:=GetTickCount;
   F_NT_LastValue:=GetNTCPUUsage;
  end; 
else 
  FActive:=False; 
end;
end;

destructor T_CpuUsage.Destroy;
begin
if FActive then begin;
   FActive:=False; 
  case FWinVersion of
  wv_9X:
   begin;
    GetReestrData(HKEY_DYN_DATA,'PerfStats\StopStat','KERNEL\CPUUsage');
   end; 
  wv_NT:
   begin;
    ReleaseNTCPUData;
   end; 
  end;
end; 
inherited;
end;

function T_CpuUsage.GetUsage: integer;
var Val: String;
  N: ^Integer;
  Interval,Ticks: Cardinal;
begin
Result:=0;
if FActive then begin;
  case FWinVersion of
  wv_9X:
   begin;
    Val:=GetReestrData(HKEY_DYN_DATA,'PerfStats\StatData','KERNEL\CPUUsage');
    if Val<>'' then begin;
     N:=@Val[1];
     Result:=N^;
    end else begin;
     Result:=0;
    end; 
   end; 
  wv_NT:
   begin;
    Ticks:=GetTickCount;
    F_NT_LastValue:=F_NT_Value;
    F_NT_Value:=GetNTCPUUsage;
    if (Ticks<=F_NT_LastTicks) then begin
     Result:=0;
    end else begin
     Interval:=Ticks-F_NT_LastTicks;
     Result:=Round((Timer100n-(F_NT_Value-F_NT_LastValue)/(Interval/Timer1s))/Timer100n*100);
    end;
    F_NT_LastTicks:=Ticks;
   end; 
  end;
end;  
end;

function T_CpuUsage.GetWinVersion: T_WinVersion;
var Ver: TOsVersionInfo;
begin
Ver.dwOSVersionInfoSize := SizeOf(Ver);
GetVersionEx(Ver);
with Ver do begin
  case dwPlatformId of
   VER_PLATFORM_WIN32s:    Result:=wv_3X;
   VER_PLATFORM_WIN32_WINDOWS: Result:=wv_9X;
   VER_PLATFORM_WIN32_NT:   Result:=wv_NT;
   else            Result:=wv_None;
  end; 
end;
end;


end.
