unit LrCPUUsage;

interface

uses
  Windows, Messages, SysUtils, Classes, ExtCtrls;

  function NtQuerySystemInformation(
    SystemInformationClass:integer;
    SystemInformation: pointer;
    SystemInformationLength: integer;    var ReturnLength: integer): word;
stdcall;  external 'ntdll.dll';

type
  PSystemBasicInformation=^TSystemBasicInformation;
  TSystemBasicInformation=record //0
    Unknown:DWORD;
    MaximumIncrement:DWORD;
    PhysicalPageSize:DWORD;
    NumberOfPhysicalPages:DWORD;
    LowestPhysicalPage:DWORD;
    HighestPhysicalPage:DWORD;
    AllocationGranularity:DWORD;
    LowestUserAddress:DWORD;
    HighestUserAddress:DWORD;
    ActiveProcessors:DWORD;
    NumberProcessors:byte;
  end;

  PPerformanceInfo=^TPerformanceInfo;   //2
  TPerformanceInfo =record
    IdleTime:LARGE_INTEGER;
    ReadTransferCount:LARGE_INTEGER;
    WriteTransferCount:LARGE_INTEGER;
    OtherTransferCount:LARGE_INTEGER;
    ReadOperationCount:DWORD;
    WriteOperationCount:DWORD;
    OtherOperationCount:DWORD;
    AvailablePages:DWORD;
    TotalCommittedPages:DWORD;
    TotalCommitLimit:DWORD;
    PeakCommitment:DWORD;
    PageFaults:DWORD;
    WriteCopyFaults:DWORD;
    TransistionFaults:DWORD;
    Reserved1:DWORD;
    DemandZeroFaults:DWORD;
    PagesRead:DWORD;
    PageReadIos:DWORD;
    Reserved2: array [0..2] of DWORD;
    PagefilePagesWritten:DWORD;
    PagefilePageWriteIos:DWORD;
    MappedFilePagesWritten:DWORD;
    MappedFilePageWriteIos:DWORD;
    PagedPoolUsage:DWORD;
    NonPagedPoolUsage:DWORD;
    PagedPoolAllocs:DWORD;
    PagedPoolFrees:DWORD;
    NonPagedPoolAllocs:DWORD;
    NonPagedPoolFrees:DWORD;
    TotalFreeSystemPtes:DWORD;
    SystemCodePage:DWORD;
    TotalSystemDriverPages:DWORD;
    TotalSystemCodePages:DWORD;
    SmallNonPagedLookasideListAllocateHits:DWORD;
    SmallPagedLookasideListAllocateHits:DWORD;
    Reserved3:DWORD;
    MmSystemCachePage:DWORD;
    PagedPoolPage:DWORD;
    SystemDriverPage:DWORD;
    FastReadNoWait:DWORD;
    FastReadWait:DWORD;
    FastReadResourceMiss:DWORD;
    FastReadNotPossible:DWORD;
    FastMdlReadNoWait:DWORD;
    FastMdlReadWait:DWORD;
    FastMdlReadResourceMiss:DWORD;
    FastMdlReadNotPossible:DWORD;
    MapDataNoWait:DWORD;
    MapDataWait:DWORD;
    MapDataNoWaitMiss:DWORD;
    MapDataWaitMiss:DWORD;
    PinMappedDataCount:DWORD;
    PinReadNoWait:DWORD;
    PinReadWait:DWORD;
    PinReadNoWaitMiss:DWORD;
    PinReadWaitMiss:DWORD;
    CopyReadNoWait:DWORD;
    CopyReadWait:DWORD;
    CopyReadNoWaitMiss:DWORD;
    CopyReadWaitMiss:DWORD;
    MdlReadNoWait:DWORD;
    MdlReadWait:DWORD;
    MdlReadNoWaitMiss:DWORD;
    MdlReadWaitMiss:DWORD;
    ReadAheadIos:DWORD;
    LazyWriteIos:DWORD;
    LazyWritePages:DWORD;
    DataFlushes:DWORD;
    DataPages:DWORD;
    ContextSwitches:DWORD;
    FirstLevelTbFills:DWORD;
    SecondLevelTbFills:DWORD;
    SystemCalls:DWORD;
  end;

  PSystemTimeInfo=^TSystemTimeInfo;  //3
  TSystemTimeInfo=record
    KeBootTime:LARGE_INTEGER;
    KeSystemTime:LARGE_INTEGER;
    ExpTimeZoneBias:LARGE_INTEGER;
    ExpTimeZoneId:DWORD;
    Unused:DWORD;
  end;                                                             


type
  TCPUUsageChangeEvent = procedure(Sender: TObject; CPUUsage, CPUUsageMax:real)
of object;

  TLrCPUUsage = class (TComponent)
  private
    FTimer:TTimer;
    FMaxCPUUsage:real;
    FOnCPUUsageChange: TCPUUsageChangeEvent;
    OldKeSystemTime: LARGE_INTEGER;
    OldIdleTime: LARGE_INTEGER;
    procedure CPUUsageChange(CPUUsage, CPUUsageMax:real); dynamic;
    procedure OnTime(Sender:TObject);
    procedure SetCPUUsageRefreshInterval(const Value: integer);
    function GetCPUUsageRefreshInterval: integer;
    { Private declarations }
  protected
    { Protected declarations }
  public
    function GetCPUUsage: real;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Run;
    procedure Stop;
    { Public declarations }
  published
    property OnCPUUsageChange: TCPUUsageChangeEvent read FOnCPUUsageChange
write FOnCPUUsageChange;
    property CPUUsageRefreshInterval : integer read GetCPUUsageRefreshInterval
write SetCPUUsageRefreshInterval;
    { Published declarations }
  end;

procedure Register;

implementation

//uses NTQuery;

procedure Register;
begin
  RegisterComponents('LrComp', [TLrCPUUsage]);
end;

{ TLrCPUUsage }

procedure TLrCPUUsage.CPUUsageChange(CPUUsage,CPUUsageMax: real);
begin
  if Assigned(FOnCPUUsageChange) then
FOnCPUUsageChange(Self,CPUUsage,CPUUsageMax);
end;

constructor TLrCPUUsage.Create(AOwner: TComponent);
begin
  inherited;
  FTimer:=TTimer.Create(self);
  with FTimer do
  begin
    OnTimer:=OnTime;
  end;
end;

destructor TLrCPUUsage.Destroy;
begin
  FTimer.Free;
  inherited;
end;

procedure TLrCPUUsage.OnTime(Sender: TObject);
begin
  CPUUsageChange(GetCPUUsage,FMaxCPUUsage);
end;

function TLrCPUUsage.GetCPUUsage: real;
var
  QSiResult:integer;
  SBI: TSystemBasicInformation;
  PI:TPerformanceInfo;
  STI:TSystemTimeInfo;
  r:double;
  dbSystemTime: Double;
  dbIdleTime: Double;

  function Li2Double(x: LARGE_INTEGER): Double;
  begin
    Result := x.HighPart * 4.294967296E9 + x.LowPart
  end;
begin
  Result:=0;
  try
    NtQuerySystemInformation(0, @SBI, SizeOf(SBI),QSiResult);
    NtQuerySystemInformation(2, @PI, SizeOf(PI),QSiResult);
    NtQuerySystemInformation(3, @STI, SizeOf(STI),QSiResult);
    if OldKeSystemTime.QuadPart<>0 then
    begin
      dbIdleTime := Li2Double(pi.IdleTime) - Li2Double(OldIdleTime);
      dbSystemTime := Li2Double(sti.KeSystemTime) - Li2Double(OldKeSystemTime);
      if dbSystemTime<>0 then
        dbIdleTime := dbIdleTime / dbSystemTime;
      r:=100.0 - dbIdleTime * 100.0 / SBI.NumberProcessors + 0.5;
      if r>100 then r:=100;
      Result:= r;
      if FMaxCPUUsage<Result then
        FMaxCPUUsage:=Result;
    end;
    OldIdleTime:=pi.IdleTime;
    OldKeSystemTime:=sti.KeSystemTime;
  finally
  end;
end;

function TLrCPUUsage.GetCPUUsageRefreshInterval: integer;
begin
  Result:=FTimer.Interval;
end;

procedure TLrCPUUsage.Run;
begin
  FTimer.Enabled:=True;
end;

procedure TLrCPUUsage.SetCPUUsageRefreshInterval(const Value: integer);
begin
  FTimer.Interval:=Value;
end;

procedure TLrCPUUsage.Stop;
begin
  FTimer.Enabled:=False;
end;

end.
