unit UMCMBM;

interface
uses windows, sysutils;

type TBusType=(
    btISA = 0,
    btSMBus = 1,
    btVIA686ABus = 2,
    btDirectIO = 3
    );

type TSMBType=(
    smtSMBIntel = 0,
    smtSMBAMD = 1,
    smtSMBALi = 2,
    smtSMBNForce = 3,
    smtSMBSIS = 4
);

type TSensorType=(
    stUnknown = 0,
    stTemperature = 1,
    stVoltage = 2,
    stFan = 3,
    stMhz = 4,
    stPercentage = 5
);


Type TSharedSensor=record
      ssType :TSensorType;                      // type of sensor
      ssName :array [0..11] of AnsiChar;        // name of sensor
      ssPad1 :array [0..2] of Char;              // padding of 3 byte
      ssCurrent :Double;                               //: Double;                           // current value
      ssLow :Double;                                   //: Double;                           // lowest readout
      ssHigh :Double;                                  //: Double;                           // highest readout
      ssCount :LongInt;                          // total number of readout
      ssPad2 :array [0..3] of Char;              // padding of 4 byte
      ssTotal :Extended;                         // total amout of all readouts
      ssPad3 :array [0..5] of Char;              // padding of 6 byte
      ssAlarm1 :Double;                                //: Double;                           // temp & fan: low alarm; voltage: % off;
      ssAlarm2 :Double;                                //: Double;                           // temp: high alarm
End;

type TSharedIndex=record
      iType:TSensorType;                       //     ': TSensorType;                          // type of sensor
      Count:Integer;                             //     ': integer;                              // number of sensor for that type
end;

Type TSharedInfo=record
      siSMB_Base:  Word;                      // SMBus base address
      siSMB_Type :TBusType;                  // SMBus/Isa bus used to access chip
      siSMB_Code :TSMBType;                  // SMBus sub type, Intel, AMD or ALi
      siSMB_Addr :Byte;                                //: Byte;                      // Address of sensor chip on SMBus
      siSMB_Name :array [0..40] of AnsiChar; // Nice name for SMBus
      siISA_Base : Word;                      // ISA base address of sensor chip on ISA
      siChipType :Integer;                   // Chip nr, connects with Chipinfo.ini
      siVoltageSubType :Byte;                          //: Byte;                      // Subvoltage option selected
End;

Type TSharedData=record
      sdVersion :Double;                               //: Double;                           // version number (example: 51090)
      sdIndex: array [0..9]   of TSharedIndex;   // Sensor index
      sdSensor: array [0..99]  of TSharedSensor;  // sensor info
      sdInfo : TSharedInfo;                      // misc. info
      sdStart : array [0..40]  of AnsiChar;       // start time
      sdCurrent : array [0..40]  of AnsiChar;       // current time
      sdPath : array [0..255] of AnsiChar;       // MBM path
End;

var
  Temperature       : array [1..10,1..3]  of Double;
  Voltage           : array [1..10,1..3]   of Double;
  Fan               : array [1..10,1..3]   of Double;
  cpu               : array [1..10,1..3]   of Double;
  Tempname:array[1..10]of string;
  volname:array[1..10]of string;
  fanname:array[1..10]of string;
  cpuname:array[1..10]of string;
function ReadMBM: boolean;

implementation
function ReadMBM: boolean;
var
  PSH          : ^TSharedData;
  X            : Integer;
  hSData       : Integer;
  st,sv,sf,sc,i:integer;
begin
  Result:=False;

  hSData:=OpenFileMapping(FILE_MAP_READ, False, '$M$B$M$5$S$D$');

  if hSData=0 then Exit;

  PSH:=MapViewOfFile(hSData, FILE_MAP_READ, 0, 0, 0);

  if PSH=Nil then
  begin
    CloseHandle(hSData);
    Exit;
  end;

  with PSH^ do
  begin
    st:=0;
    sv:=0;
    sf:=0;
    sc:=0;
    for X:=1 to 10 do
    for i:=1 to 3 do begin
        Temperature[X,i]:=0;
        Voltage[X,i]:=0;
        Fan[X,i]:=0;
        tempname[x]:='';
        volname[x]:='';
        fanname[x]:='';
        cpuname[x]:='';
        cpu[x,i]:=0;
    end;
    for i:=0 to 99 do
        with psh^.sdSensor[i] do
        case ssType of
            stTemperature:if st<10 then begin
                inc(st);
                Temperature[st][1]:=ssCurrent;
                Temperature[st][2]:=ssLow;
                Temperature[st][3]:=ssHigh;
                tempname[st]:=ssname;
            end;
            stVoltage:if sv<10 then begin
                inc(sv);
                Voltage[sv][1]:=ssCurrent;
                Voltage[sv][2]:=ssLow;
                Voltage[sv][3]:=ssHigh;
                volname[sv]:=ssname;
            end;
            stFan:if sf<10 then begin
                inc(sf);
                fan[sf][1]:=ssCurrent;
                fan[sf][2]:=ssLow;
                fan[sf][3]:=ssHigh;
                fanname[sf]:=ssname;
            end;
            stMhz:if sc<10 then begin
                inc(sc);
                cpu[sc][1]:=ssCurrent;
                cpu[sc][2]:=ssLow;
                cpu[sc][3]:=ssHigh;
                cpuname[sc]:=ssname;
            end;
        end;
  end;

  {
Type TSharedSensor=record
      ssType :TSensorType;                      // type of sensor
      ssName :array [0..11] of AnsiChar;        // name of sensor
      ssPad1 :array [0..2] of Char;              // padding of 3 byte
      ssCurrent :Double;                               //: Double;                           // current value
      ssLow :Double;                                   //: Double;                           // lowest readout
      ssHigh :Double;                                  //: Double;                           // highest readout
      ssCount :LongInt;                          // total number of readout
      ssPad2 :array [0..3] of Char;              // padding of 4 byte
      ssTotal :Extended;                         // total amout of all readouts
      ssPad3 :array [0..5] of Char;              // padding of 6 byte
      ssAlarm1 :Double;                                //: Double;                           // temp & fan: low alarm; voltage: % off;
      ssAlarm2 :Double;                                //: Double;                           // temp: high alarm
End;







  }
  UnMapViewOfFile(PSH);
  CloseHandle(hSData);
  Result:=True;
end;
////////////////////////////////////////

end.
