unit UMCstatistics;

interface
uses UMCaux;
const statsig='UMC1';
      statcopy='[This is binary UMC statistics file (c) Stanislav Mikov]';
      statfilename='statistics.umc';
      dpy=366;
type TuStatEntries=record
    TextSize,AttSize,HeadSize:single;
end;
type TuHour=record
    used:boolean;
    DATA:TuStatEntries;
end;

type TuDay=record
    HOURS:array[0..23] of TuHour;
end;

type TuYear=object
    DAYS:array[1..dpy] of TuDay;
    procedure Reset;
    Procedure AddValueToday(nTextSize,nAttSize,nHeadSize:single);
    procedure LoadFromFile(FileName:string);
    procedure SaveToFile(FileName:string);
    function IsDayUsed(DayNo:integer):boolean;
    function GetMaxSize:extended;
    function GetMaxSizePerDay:extended;
    function PerDayValue(Day:integer;index:integer):single;
end;

var StatData:TuYear;
implementation

uses SysUtils, DateUtils, Classes;
{ TuYear }

procedure TuYear.AddValueToday(nTextSize,nAttSize,nHeadSize:single);
var //n:tdatetime;
    day,hour:integer;
begin
    day:=DayOfTheYear(now);
    hour:=HourOfTheDay(now);
    if DAYS[day].HOURS[hour].used then begin
        DAYS[day].HOURS[hour].data.TextSize:=DAYS[day].HOURS[hour].data.TextSize+nTextSize;
        DAYS[day].HOURS[hour].data.AttSize:=DAYS[day].HOURS[hour].data.AttSize+nAttSize;
        DAYS[day].HOURS[hour].data.HeadSize:=DAYS[day].HOURS[hour].data.HeadSize+nHeadSize;
    end
    else begin
        DAYS[day].HOURS[hour].data.TextSize:=nTextSize;
        DAYS[day].HOURS[hour].data.AttSize:=nAttSize;
        DAYS[day].HOURS[hour].data.HeadSize:=nHeadSize;
        DAYS[day].HOURS[hour].used:=true;
    end;

end;

function TuYear.GetMaxSize: extended;
var i,j:integer;
    r,tr:extended;
begin
    r:=0;
    for i:=1 to dpy do
        for j:=0 to 23 do begin
            tr:=DAYS[i].HOURS[j].DATA.HeadSize+DAYS[i].HOURS[j].DATA.TextSize+DAYS[i].HOURS[j].DATA.AttSize;
            if tr>r then r:=tr;
        end;
    result:=r;


end;

function TuYear.GetMaxSizePerDay: extended;
var i:integer;
    r,tr:extended;
begin
    r:=0;
    for i:=1 to dpy do begin
//        for j:=0 to 23 do begin
            tr:=PerDayValue(i,4);
            if tr>r then r:=tr;
        end;
    result:=r;

end;

function TuYear.IsDayUsed(DayNo: integer): boolean;
var i:integer;
    b:boolean;
begin
    b:=false;
    for i:=0 to 23 do
        b:=b or DAYS[DayNo].HOURS[i].used;
    result:=b;
end;

procedure TuYear.LoadFromFile(FileName: string);
var n:TFileStream;
    temp:string[255];
    z:packed array [1..dpy] of boolean;
    i,j:integer;
begin
    Reset;
    if not fileexists(filename) then exit;
    n:=TFileStream.create(filename,fmOpenRead);
    n.Read(temp,length(statsig));
    n.Read(temp,length(statcopy));
{    for i:=1 to dpy do
        z[i]:=IsDayUsed(i);}
    n.Read(z,sizeof(z));
    for i:=1 to dpy do
        if z[i] then
        for j:=0 to 23 do
        with DAYS[i].Hours[j] do
        begin
            n.Read(used,sizeof(used));
            n.Read(data.textsize,sizeof(data.textsize));
            n.Read(data.attsize,sizeof(data.attsize));
            n.Read(data.headsize,sizeof(data.headsize));
        end;


        {begin
            n.Read(DAYS[i].Hours,sizeof(DAYS[i].Hours));
        end;}
    n.Free;
end;

function TuYear.PerDayValue(Day, index: integer): single;
var r:single;
    i:integer;
begin
    result:=0;
    if not IsDayUsed(day) then exit;
    r:=0;
    for i:=0 to 23 do
        case index of
            1:r:=r+DAYS[DAY].HOURS[i].DATA.TextSize;
            2:r:=r+DAYS[DAY].HOURS[i].DATA.AttSize;
            3:r:=r+DAYS[DAY].HOURS[i].DATA.HeadSize;
        else
            r:=r+DAYS[DAY].HOURS[i].DATA.HeadSize+DAYS[DAY].HOURS[i].DATA.AttSize+DAYS[DAY].HOURS[i].DATA.TextSize;
        end;
    result:=r;
end;

procedure TuYear.Reset;
var i,j:integer;
begin
    for i:=1 to dpy do
        for j:=0 to 23 do begin
            DAYS[i].HOURS[j].used:=false;
            DAYS[i].HOURS[j].DATA.TextSize:=0;
            DAYS[i].HOURS[j].DATA.AttSize:=0;
            DAYS[i].HOURS[j].DATA.HeadSize:=0;
        end;


end;

procedure TuYear.SaveToFile(FileName: string);
var n:TFileStream;
    z:packed array [1..dpy] of boolean;
    i,j:integer;
begin
    if fileexists(filename) then
    n:=TFileStream.create(filename,fmOpenWrite)
    else
    n:=TFileStream.create(filename,fmCreate);
    n.Write(statsig,length(statsig));
    n.Write(statcopy,length(statcopy));
    for i:=1 to dpy do
        z[i]:=IsDayUsed(i);
    n.Write(z,sizeof(z));
    for i:=1 to dpy do
        if IsDayUsed(i) then
        for j:=0 to 23 do
        with DAYS[i].Hours[j] do
        begin
            n.Write(used,sizeof(used));
            n.Write(data.textsize,sizeof(data.textsize));
            n.Write(data.attsize,sizeof(data.attsize));
            n.Write(data.headsize,sizeof(data.headsize));
        end;
    n.Free;
end;

end.
