library uc;


uses
  Windows,
  SysUtils,
  Classes;

{$E TBP}

{$R *.res}

type
  ITBPDataProvider = interface ['{9DD91B89-A551-4180-8A81-2CCF584CD4BF}']
    function GetDataByID(ID: Integer; B: Pointer; BSize: Integer): Integer; stdcall;
    function SetDataByID(ID: Integer; B: Pointer; BSize: Integer): Integer; stdcall;
    function GetIntValue(ID: Integer): Integer; stdcall;
    function SetIntValue(ID, Value: Integer): Integer; stdcall;
    function GetIDType(ID: Integer): Integer; stdcall;
    function ItemCount: Integer; stdcall;
    function ExecuteMacro(AMacro: Pointer; MaxLen: Integer; InData, OutData: ITBPDataProvider): HResult; stdcall;
  end;


procedure TBP_Initialize; stdcall;
begin

end;

procedure TBP_Finalize; stdcall;
begin

end;

function GetStr(const S: String; B: PChar; BSize: Integer): Integer;
begin
  if (B = nil) or (BSize < 0) then Result := Length(S) else
  try
    if BSize > Length(S) then BSize := Length(S);
    if BSize > 0 then Move(S[1], B[0], BSize);
    Result := BSize;
  except
    Result := 0;
  end;
end;

function TBP_GetName(B: PChar; BSize: Integer): Integer; stdcall;
begin
  Result := GetStr('UUE collector', B, BSize);
end;

function TBP_GetVersion(B: PChar; BSize: Integer): Integer; stdcall;
begin
  Result := GetStr('0.0', B, BSize);
end;

function TBP_GetStatus: Integer; stdcall;
begin
  Result := 0;
end;

function TBP_NeedConfig: Integer; stdcall;
begin
  Result := 1;
end;

function TBP_NeedCOM: Integer; stdcall;
begin
  Result := 0;
end;

function TBP_Setup: Integer; stdcall;
begin
  MessageBoxEx(GetActiveWindow, 'Configured OK!', 'My TEST Plug-in', MB_OK or MB_ICONINFORMATION, LANG_NEUTRAL);
  Result := 0;
end;

const
  midxSubject        = 1;
  midxDate           = 2;
  midxComment        = 3;
  midxInReplyTo      = 4;
  midxMessageID      = 5;
  midxNewsgroups     = 6;
  midxMailer         = 7;
  midxContentType    = 8;
  midxContentSubType = 9;
  midxExpireDate     = 10;
  midxOrganization   = 11;
  midxContentID      = 12;
  midxContentMD5     = 13;
  midxPriority       = 14;
  midxImportance     = 15;
  midxContentLocation= 16;
  midxEncoding       = 17;
  midxCharset        = 18;
  midxBoundary       = 19;
  midxMsgEncoding    = 20;
  midxC_Name         = 21;
  midxCD_Name        = 22;
  midxReportType     = 23;
  midxReferences     = 24;
  midxC_Description  = 25;
  midxContentDisposition = 26;
  midxContentLanguage = 27;
  midxC_ID            = 29;
  midxC_AccessType    = 30;
  midxC_Expiration    = 31;
  midxC_Size          = 32;
  midxC_Permission    = 33;
  midxC_Site          = 34;
  midxC_Directory     = 35;
  midxC_Mode          = 36;
  midxC_Server        = 37;
  midxC_Subject       = 38;
  midx_Files          = 39;
  midx_Msgs           = 40;
  midxReturnPath      = 41;
  midxFromName        = 42;
  midxFromAddr        = 43;
  midxReplyName       = 44;
  midxReplyAddr       = 45;
  midxToName          = 46;
  midxToAddr          = 47;
  midxServerID        = 48;
  midxRRC             = 49;
  midxRRQ             = 50;
  midxFileSubst       = 51;
  midxC_Number        = 52;
  midxC_Total         = 53;
  midxXCFile          = 54;
  midxMDN_To          = 55;
  midxMDN_Options     = 56;
  midxRefList         = 57;
  midxC_MICAlg        = 58;
  midxC_SMIMEType     = 59;
  midxC_Protocol      = 60;
  midxC_ProtocolType  = 61;
  midxC_ProtocolSubType=62;
  midxMatter          = 63;
  midxListHelp        = 64;
  midxListUnsub       = 65;
  midxListSub         = 66;
  midxListPost        = 67;
  midxListOwner       = 68;
  midxListArchive     = 69;
  midxDecodedFrom     = 70;
  midxDecodedTo       = 71;
  midxDecodedSubj     = 72;
  midxDecodedToEtc    = 73;

  mpidMessageHeader            =  100000;   // raw message RFC 822 header
  mpidMessageBody              =  100001;   // decoded message body (not available yet)
  mpidMessageAttachments       =  100002;   // a CR-delimited list of attachments
  mpidMessageSender            =  100003;   // decoded message sender (FROM)
  mpidMessageSubject           =  100004;   // decoded message subject
  mpidRawMessage               =  100005;   // raw message body


type
  TBPGetDataProc = function(ID, DataID: Integer; Buf: PChar; BufSize: Integer): Integer; stdcall;

function GetDataStr(Idx, MsgID: Integer; GetData: TBPGetDataProc): String;
var
  I, J: Integer;
  P: PChar;
  Sz: Integer;
begin
  P := nil;
  Sz := 0;
  Result := '';
  try
    I := GetData(MsgID, Idx, nil, 0);
    if I > 0 then
    begin
      if I > Sz then
      begin
        ReallocMem(P, I);
        Sz := I;
      end;
      I := GetData(MsgID, Idx, P, Sz);
      if I > 0 then SetString(Result, P, I);
    end;
  except
    Result := '';
  end;
  FreeMem(P);
end;

function TBP_GetSpamScore(MsgID: Integer; GetData: TBPGetDataProc): Integer; stdcall;
  var
    L: TStringList;
    S: String;
begin
  Result := 0;
  try
    S := Trim(UpperCase(GetDataStr(mpidMessageSubject, MsgID, GetData)));
    // this is just a sample, it is possible to use virtually all data from a message to
    // analyze it!
    if Copy(S, 1, 4) = 'ADV:' then Result := 70;
    if (Pos('TEEN', S) > 0) or (Pos('PORN', S) > 0) or (Pos('ADULT', S) > 0) or (Pos('LOANS', S) > 0) or (Pos('(GET CASH', S) > 0)
       or (Pos('ADV:', S) > 0) or (Pos('ENLARGE YOUR', S) > 0)
       or (Pos('MORTGAGE', S) > 0)  then Result := 80;
  except
  end;
end;


// TBP_FeedSpam is called when the "Specials"|Mark as Junk (or NOT Junk)" menu 
// command is used. The value of IsSPAM is non-zero if "Mark as Junk" was used,
// if IsSPAM is zero, "Mark as NOT Junk" was used.
// The function should return zero if the message was processed (well, it does
// not really matter since the message was passed to the plug-in anyway :-)

function TBP_FeedSpam(MsgID, IsSPAM: Integer; GetData: TBPGetDataProc): Integer; stdcall;
var S: String;
begin
  S := UpperCase(GetDataStr(mpidMessageBody, MsgID, GetData));
  Result := Pos('HELLO', S);
end;


// TBP_GetMacroList returns the CR/LF-separated list of supported macros,
// like 'MyMacro'#13#10'MySuperMacro'#13#10'MyDuperMacro'

function TBP_GetMacroList(AData: Pointer; ADataSize: Integer): Integer; stdcall;
begin
  Result := GetStr('TESTParam', AData, ADataSize);
end;


// Here is the current list of template object indexes
// Their meaning should be clear from the name :-)

const
  tpxQuotePrefix       = 200;
  tpxWrapJustify       = 201;
  tpxClear             = 202;
  tpxIsSignature       = 203;
  tpxSignComplete      = 204;
  tpxEncryptComplete   = 205;
  tpxUseSMIME          = 206;
  tpxUsePGP            = 207;
  tpxRCR               = 208;
  tpxRRQ               = 209;
  tpxSplit             = 210;
  tpxCharset           = 211;
  tpxPriority          = 212;
  tpxAttachVCard       = 213;
  tpxAccount           = 214;
  tpxFrom              = 215;
  tpxReplyTo           = 216;
  tpxReturnPath        = 217;
  tpxTo                = 218;
  tpxCC                = 219;
  tpxBCC               = 220;
  tpxOrg               = 221;
  tpxSubject           = 222;
  tpxFullSubject       = 223;
  tpxComment           = 224;
  tpxOldTo             = 225;
  tpxOldFrom           = 226;
  tpxOldReplyTo        = 227;
  tpxOldCC             = 228;
  tpxOldBCC            = 229;
  tpxOldSubject        = 230;
  tpxOldComment        = 231;
  tpxMatter            = 232;
  tpxOldMatter         = 233;
  tpxMsgID             = 234;
  tpxOldMsgID          = 235;
  tpxOldDate           = 236;
  tpxOldRcvDate        = 237;
  tpxOldReturn         = 238;
  tpxOldOrg            = 239;
  tpxOldText           = 240;
  tpxText              = 241;
  tpxSingleRe          = 242;
  tpxHeaders           = 243;
  tpxAttachments       = 244;
  tpxOldAttachments    = 245;
  tpxCookieCount       = 246;
  tpxOldCharset        = 247;
  tpxTotalPages        = 248;
  tpxCurrentPage       = 249;
  tpxTracking          = 248;
  tpxQuoteStyle        = 249;
  tpxTextLF            = 250;
  tpxRegExpPattern     = 251;
  tpxRegExpText        = 252;



function TBP_ExecMacro(AMacro: Pointer; MaxLen: Integer; Template, Params: ITBPDataProvider): Integer; stdcall;
var
  S, P: String;
  I, C: Integer;
  B: PChar;
  Sz, BSz: Integer;
begin
  Result := -2;
  B := nil;
  BSz := 0;
  if (MaxLen > 0) then
  try
    SetString(S, PChar(AMacro), MaxLen);
    if UpperCase(S) = 'TESTPARAM' then
    begin
      P := '';
      C := Params.ItemCount-1;
      for I := 0 to C do
      begin
        Sz := Params.GetDataByID(I, nil, 0);
        if Sz > BSz then
        begin
          ReallocMem(B, Sz);
          BSz := Sz;
        end;
        Sz := Params.GetDataByID(I, B, BSz);
        SetString(S, B, Sz);
        if P <> '' then P := P + ' AND ';
        P := P + S;
      end;
      if P = '' then P := '<none>';
      Result := 213213;
      Template.SetDataByID(Result, PChar(P), Length(P));
    end;
  finally
    FreeMem(B);
  end;
end;


exports TBP_NeedConfig, TBP_FeedSpam, TBP_GetStatus, TBP_GetVersion, TBP_GetName,
  TBP_Finalize, TBP_Initialize, TBP_Setup, TBP_NeedCOM, TBP_GetSpamScore,
  TBP_GetMacroList, TBP_ExecMacro;

begin
end.
